#include <SFML/Graphics.hpp>
#include <vector>
#include <cmath>
#include "gcalc.h"

using namespace sf;

//------------------------------------------------------------------------------
/**Function sine {sin,0,100,Point{20,150},1000,50,50}; // sine curve
// plot sin() in the range [0:100) with (0,0) at (20,150)
// using 1000 points; scale x values *50, scale y values *50
*/
class Function : public Drawable {
public:
    Function( float begiN, float enD, Vector2f s, const int n, float xS, float yS, std::string& fstrg = "sin(x);");
    Vertex* vertices;// = new Vertex[number_of_points];
    void draw(RenderTarget& target, RenderStates states) const;
    int getNoP() { return number_of_points; };
    ~Function() { free(vertices); vertices = NULL; } // DON'T FORGET TO FREE UP SPACE!
private:
    float startx;
    float stopx;
    Vector2f start;
    const int number_of_points;
    float xscale;
    float yscale;
};

Function::Function(float begiN, float enD, Vector2f s, const int n, float xS, float yS, std::string& fstrng)
  : startx(begiN), stopx(enD), start(s), number_of_points(n),
    xscale(xS), yscale(yS), vertices(new Vertex[number_of_points])
{

    float deltaX = (stopx-startx)/number_of_points, x = startx;
    //initialize x as xstart in the names table.
    //set fvals[0] = calc(x);
    //loop with increments
    vertices[0] = Vertex(Vector2f(start.x,start.y+yscale*sin(x)));
    for(int i = 1; i < number_of_points; ++i) {
        x += deltaX;
        vertices[i] = Vertex(Vector2f(x,start.y+yscale*sin(x)));
    }
}

void Function::draw(RenderTarget& target, RenderStates states) const {
    target.draw(vertices, number_of_points, LinesStrip, states);
}

//------------------------------------------------------------------------------

class Axis : public Drawable {
public:

    Axis(int d, Vertex xy, int l, int n=0);
    //std::vector<Vertex> vertices;
    Vertex vertices[22];

    void draw(RenderTarget& target, RenderStates states) const;
private:
    //enum Orientation { x, y };
    Vertex start;
    int length;
    int number_of_notches;

};

Axis::Axis(int d, Vertex xy, int l, int n)
{
    switch (d)
    {
    case 0:  //Axis::x:
    {
        xy.color= Color(255,255,0);
        vertices[0] = xy;
        vertices[1] = Vertex(Vector2f(xy.position.x+l,xy.position.y),Color(255,255,0));
        // axis line
        if (1<n)  // add notches
        {
            int deltaX = l/n;
            int x = xy.position.x+deltaX;  // x-coord of first notch
            for (int i = 0; i<2*n; i+=2)
            {
                vertices[2+i] =  Vertex(Vector2f(x,xy.position.y),Color(255,255,0));
                vertices [3+i] = Vertex(Vector2f(x,xy.position.y-10),Color(255,255,0));
                x += deltaX;
            }
        }
//        // label under the line
//        label.move(length/3,xy.y+20);
        break;
    }
    case 1:  //Axis::y:
    {
        xy.color= Color(255,255,0);
        vertices[0] = xy;
        vertices[1] = Vertex(Vector2f(xy.position.x,xy.position.y-l),Color(255,255,0));

        if (1<n)        // add notches
        {
            int deltaY = l/n;
            int y = xy.position.y-deltaY;  // x-coord of first notch
            for (int i = 0; i<2*n; i+=2)
            {
                vertices[2+i] = Vertex(Vector2f(xy.position.x,y),Color(255,255,0));
                vertices[3+i] = Vertex(Vector2f(xy.position.x+10,y),Color(255,255,0));
                y -= deltaY;
            }
        }
        break;
    }
    }
}

void Axis::draw(RenderTarget& target, RenderStates states) const {
    target.draw(vertices, 22, Lines, states);
}
